@@ -23,7 +23,6 @@ use {Config, Action, distances, cost, pair_cost, NodeID, Coords, Node, random_fg
 
 pub struct Screen {
     pub max_id: u64,
-
     pub nodes: HashMap<NodeID, Node>,
     pub arrows: Vec<(NodeID, NodeID)>,
     pub work_path: Option<String>,
@@ -353,8 +352,11 @@ impl Screen {
         // map an alphanumeric char to each candidate NodeID
         let mapping: HashMap<&str, NodeID> =
             chars.split("").skip(1).zip(nodes.into_iter()).collect();
+
+        // clear the prompt
+        print!("{}{}", cursor::Goto(1, self.dims.1), clear::AfterCursor);
+
         // print the hilighted char at each choice
-        self.draw();
         for (&c, &node_id) in &mapping {
             let &coords = self.drawn_at(node_id).unwrap();
             let (x, y) = self.internal_to_screen_xy(coords).unwrap();
@@ -662,7 +664,7 @@ impl Screen {
     }
 
     fn delete_recursive(&mut self, node_id: NodeID) {
-        debug!("deleting node {}", node_id);
+        trace!("delete_recursive({})", node_id);
         if let Some(node) = self.nodes.remove(&node_id) {
             // clean up any arrow state
             self.arrows.retain(|&(ref from, ref to)| from != &node_id && to != &node_id);
@@ -676,8 +678,8 @@ impl Screen {
     fn delete_selected(&mut self, reselect: bool) {
         trace!("delete_selected()");
         if let Some(selected_id) = self.selected.take() {
+            let (_, height) = self.drawable_subtree_dims(selected_id).unwrap();
             let coords = self.drawn_at.remove(&selected_id);
-            debug!("coords: {:?}", coords);
             // remove ref from parent
             if let Some(parent_id) = self.parent(selected_id) {
                 trace!("deleting node {} from parent {}", selected_id, parent_id);
@@ -686,11 +688,9 @@ impl Screen {
             }
             // remove children
             self.delete_recursive(selected_id);
-            if let Some(c) = coords {
+            if let Some((x, y)) = coords {
                 if reselect {
-                    // need to draw here or there will be nothing to click_select below
-                    self.draw();
-                    self.click_select(c);
+                    self.click_select((x, y + height));
                 }
             }
         }
@@ -761,7 +761,21 @@ impl Screen {
     }
 
     fn create_sibling(&mut self) {
-        if let Some(selected_id) = self.selected {
+        if let Some(mut selected_id) = self.selected {
+            if self.with_node(selected_id, |n| n.content.is_empty()).unwrap() {
+                // we just hit enter twice, so go back a level
+                let sel_parent = self.parent(selected_id).unwrap();
+                if sel_parent == self.drawing_root {
+                    // don't want to create a sibling of the drawing root
+                    // because that's not underneath the drawing root
+                    self.unselect();
+                    return;
+                }
+                self.select_node(sel_parent);
+                selected_id = sel_parent;
+            }
+            let selected_id = selected_id;
+
             if let Some(parent_id) = self.parent(selected_id) {
                 if parent_id == self.drawing_root {
                     // don't want to deal with this case right now
@@ -771,7 +785,14 @@ impl Screen {
 
                 self.with_node_mut(node_id, |node| node.parent_id = parent_id);
                 let added = self.with_node_mut(parent_id, |parent| {
-                    parent.children.push(node_id);
+                    // it's possible that selected_id has been deleted by now
+                    // due to it being empty when we entered the function
+                    // (double enter for going up a level)
+                    let idx = parent.children
+                        .iter()
+                        .position(|&e| e == selected_id)
+                        .unwrap_or(0);
+                    parent.children.insert(idx + 1, node_id);
                 });
                 if added.is_some() {
                     self.select_node(node_id);
@@ -1111,21 +1132,18 @@ impl Screen {
         trace!("select_node({})", node_id);
         self.unselect();
         if node_id != 0 {
-            self.with_node_mut(node_id, |mut node| node.selected = true);
-            self.selected = Some(node_id);
-
-            // draw() needed to make visible / scroll accurate
-            self.draw();
-
-            if let Some(visible) = self.node_is_visible(node_id) {
-                if !visible {
-                    self.scroll_to_selected();
-                }
-            }
+            // it's possible that unselecting above actually caused
+            // this node to be deleted, due to its parent (previous
+            // selection) being empty.  To account for this, we need
+            // to only set self.selected to node_id if the with_node
+            // succeeds.
+            self.with_node_mut(node_id, |mut node| node.selected = true)
+                .map(|_| self.selected = Some(node_id));
         }
     }
 
     fn click_screen(&mut self, coords: Coords) {
+        trace!("click_screen({:?})", coords);
         if coords.0 > self.dims.0 || coords.1 > self.view_y + self.dims.1 {
             warn!("click way off-screen");
             return;
@@ -1171,7 +1189,7 @@ impl Screen {
             });
         }
 
-        // no parent loops
+        // no loops, no orphans
         debug!("testing that 0 is the ancestor of all nodes");
         for &node_id in self.nodes.keys() {
             assert!(self.is_parent(0, node_id));
@@ -1263,11 +1281,15 @@ impl Screen {
         trace!("draw()");
         print!("{}", clear::All);
 
-        self.draw_header();
-
         // print visible nodes
         self.draw_children_of_root();
 
+        // TODO figure out why header doesn't get shown
+        // when a root node is NOT drawn at 1,1
+        // (this only happens when draw_header() is above
+        // the call to draw_children_of_root()...
+        self.draw_header();
+
         // print logs
         if self.show_logs && self.dims.0 > 4 && self.dims.1 > 7 {
             let mut sep = format!("{}{}logs{}",
@@ -1321,6 +1343,9 @@ impl Screen {
 
         print!("{}", cursor::Hide);
         self.flush();
+
+        // TODO evaluate where the best place to do this is
+        self.scroll_to_selected();
     }
 
     fn draw_scrollbar(&self) {
@@ -1381,8 +1406,7 @@ impl Screen {
         let mut pre_meta = String::new();
         let mut buf = String::new();
         // only actually print it if we're in-view
-        if let Some(screen_coords) = self.internal_to_screen_xy(internal_coords) {
-            let (x, y) = screen_coords;
+        if let Some((x, y)) = self.internal_to_screen_xy(internal_coords) {
             write!(pre_meta, "{}{}", cursor::Goto(x, y), color).unwrap();
             if node.selected {
                 write!(&mut pre_meta, "{}", style::Invert).unwrap();
@@ -1424,11 +1448,9 @@ impl Screen {
                 buf.push('…');
             }
 
-            print!("{}", buf);
+            print!("{}{}", buf, style::Reset);
         }
 
-        print!("{}", style::Reset);
-
         let visible = buf.replace(reset, "").replace(&*pre_meta, "");
         let visible_graphemes = UnicodeSegmentation::graphemes(&*visible, true).count();
         self.drawn_at.insert(node_id, internal_coords);
